#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
from os.path import join as buildpath
from pentest.manager import JarvisManager
from pentest.utils import JarvisLogger
from datetime import datetime
from slugify import slugify
import sys
import subprocess


class JarvisCmd(object):
	"""Top-level Jarvis class that manages penetration tests"""
	def __init__(self, *args):
		self.args = list(*args)
		self.name = os.path.basename(self.args[0])
		self.manager = JarvisManager()
		self.logger = JarvisLogger(self.manager)
		self.args_config = None
		self.record = False
		self._output_directory = self.manager.outdir

		self.clean_env = os.environ.copy()
		dirs = self.clean_env['PATH'].split(':')
		dir_to_exclude = os.path.dirname(self.args[0])
		del dirs[dirs.index(dir_to_exclude)]
		self.clean_env['SHELL'] = '/bin/bash'
		self.clean_env['PATH'] = ':'.join(dirs)

	def __str__(self):
		return ' '.join(map(lambda x: "'%s'" % x, self.args))

	def __repr__(self):
		return ' '.join(self.args)

	@property
	def output_directory(self):
		if self.manager.current:
			return buildpath(self.manager.current, self.manager.userdir, self._output_directory)
		else:
			return False

	@output_directory.setter
	def output_directory(self, value):
		self._output_directory = value

	def set_args_config(self, args_config):
		self.args_config = args_config

	def set_recordable(self, recordable):
		self.record = recordable

	def run(self, marker=None):
		if not self.manager.current:
			self.args[0] = self.name
			self.execute()
		elif '--nojarvis' in self.args:
			self.args[0] = self.name
			del self.args[self.args.index('--nojarvis')]
			self.execute()
		else:
			for passthrough_arg in self.args_config['passthrough_args']:
				if passthrough_arg in self.args:
					self.args[0] = self.name
					self.execute()
					break

			self.marker = marker
			self.prepare_command()
			self.logger.register(str(self))
			if self.record:
				JarvisRecordableCommand(self).run()
			else:
				self.execute()

	def prepare_command(self):
		self.args[0] = self.name

		# if no marker is provided, hook will not run
		if self.marker:
			for output_arg in self.args_config['output_args']:
				self.add_arguments(
					output_arg.get('pos', None) or 'begin',
					output_arg['arg'],
					output_arg.get('filename', None) or	self.generate_file_name(
						output_arg['format'],
						self.marker
					)
				)

	def generate_file_name(self, format_, marker, record=False):
		filename = self.name + '-'
		filename += (marker.replace('/', '_').replace(':', '_') + '-') if marker else ''
		filename += self.timestamp()
		filename += ('.' + format_) if format_ else ''
		return buildpath(self.output_directory, filename)

	def timestamp(self):
		return slugify(unicode(str(datetime.now().replace(microsecond=0))))

	def execute(self):
		"""
		Execute the original program by patching the PATH
		environment variable
		"""
		subprocess.call(self.args, env=self.clean_env)
		sys.exit(0)

	def add_arguments(self, pos, *args):
		"""
		Method used to add safely add arguments into the commandline
		"""
		if pos == 'begin':
			for arg in args[::-1]:
				self.args.insert(1, arg)
		elif pos == 'end':
			for arg in args:
				self.args.insert(len(self.args), arg)
		else:
			print '[!] Invalid argument position : %s' % pos
			sys.exit(0)


class JarvisRecordableCommand(object):

	def __init__(self, JarvisCommand):
		self.cmd = JarvisCommand
		self.recordable_cmd = ['script', '-e', '-q', '-c']
		self.recordable_cmd.append(str(self.cmd))
		self.recordable_cmd.append(self.record_filename)

	@property
	def record_filename(self):
		filename = self.cmd.name + '-'
		filename += 'record-'
		filename += (self.cmd.marker.replace('/', '_').replace(':', '_') + '-') if self.cmd.marker else ''
		filename += self.cmd.timestamp()
		filename += '.txt'
		return buildpath(self.cmd.output_directory, filename)

	def run(self):
		subprocess.call(self.recordable_cmd, env=self.cmd.clean_env)
		sys.exit(0)



from pentest.command import JarvisCmd
import sys
__name__ = "sslyze"


def main():
	args_config = {
		'passthrough_args': ['--xml_out'],
		'output_args': [
			{
				'format': 'xml',
				'arg': '--xml_out'
			}
		]
	}

	cmd = JarvisCmd(sys.argv)
	cmd.set_args_config(args_config)

	for arg in cmd.args:
		if 'targets_in' in arg:
			target = 'multiple-hosts'
			break
	else:
		target = cmd.args[-1].replace('/', '_')

	cmd.run(target)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pentest.parser import JarvisParser
import ConfigParser
import json
import os
from os.path import join as buildpath
from os.path import expanduser
import argparse


class JarvisManager(object):
	"""Top-level Jarvis class that manages penetration tests"""
	def __init__(self):
		config = ConfigParser.ConfigParser()
		config.read("/etc/jarvis.conf")
		self.userdir = config.get('jarvis', 'user_directory')
		self.outdir = config.get('jarvis', 'output_directory')
		self.histfile = config.get('jarvis', 'pentests_history')
		self.tree = json.loads(config.get('jarvis', 'pentest_tree'))
		self.notes_ = config.get('jarvis', 'notesfiles')
		if not os.path.isfile(self.histfile):
			open(self.histfile, 'w').close()
		self.fd = open(self.histfile, 'rb+')

		self.update()
		self.parser = argparse.ArgumentParser("Jarvis")
		subparsers = self.parser.add_subparsers(dest='command', help='Action to perform')
		start_parser = subparsers.add_parser('start', help='Edits a picture')
		start_parser.add_argument('directory', help='The directory name')
		subparsers.add_parser('stop', help='Stop the current pentest')
		subparsers.add_parser('init', help='Creates working tree for current pentest')
		subparsers.add_parser('screenshot', help='Takes a screenshot')
		subparsers.add_parser('notes', help='Starts notes edition')
		subparsers.add_parser('img', help='Lists screenhosts')

	def update(self):
		self.fd.seek(0, os.SEEK_END)
		if self.fd.tell() == 0:
			self.current = None
		else:
			while True:
				self.fd.seek(-1, os.SEEK_CUR)
				char = self.fd.read(1)
				self.fd.seek(-1, os.SEEK_CUR)
				if char == "\n" or self.fd.tell() == 0:
					if char == "\n":
						self.fd.read(1)
					self.current = self.fd.read()
					break
		self.fd.seek(0, os.SEEK_SET)

	def shortcutize(self):
		try:
			os.unlink("%s/pentest" % expanduser("~"))
		except Exception:
			pass
		os.symlink(self.current, "%s/pentest" % expanduser("~"))

	def start(self, path):
		self.fd.seek(0, os.SEEK_END)
		if os.path.isdir(path):
			if os.path.getsize(self.histfile) == 0:
				self.fd.write("%s" % os.path.abspath(path))
			else:
				self.fd.write("\n%s" % os.path.abspath(path))
		self.fd.seek(0, os.SEEK_SET)
		self.current = os.path.abspath(path)
		self.shortcutize()
		print '[i] A pentest located at %s has been stacked' % os.path.abspath(path)

	def stop(self):
		self.fd.seek(0, os.SEEK_END)
		if self.fd.tell() != 0:
			while True:
				self.fd.seek(-1, os.SEEK_CUR)
				char = self.fd.read(1)
				self.fd.seek(-1, os.SEEK_CUR)
				if char == '\n' or self.fd.tell() == 0:
					self.fd.truncate()
					break
		self.fd.seek(0, os.SEEK_SET)
		self.update()
		self.shortcutize()

	def init(self):
		if self.current:
			for directory in self.tree:
				if not os.path.isdir(os.path.join(self.current, directory)):
					os.mkdir(os.path.join(self.current, directory))
				else:
					print '[!] %s already exists, skipping' % os.path.join(self.current, directory)

	def screenshot(self):
		screenshot_id = str(uuid.uuid4())
		subprocess.check_output(["import", "%s/%s/img/%s.png" % (self.current, self.userdir, screenshot_id)])
		screenshot_name = subprocess.check_output(["zenity", "--entry", "--title=Jarvis-Screen", "--text=Nom de la capture d'écran"])
		screenshot_name = screenshot_name[:-1]
		while True:
			if os.path.isfile(os.path.join(self.current, self.userdir, "img", "%s.png" % screenshot_name)):
				screenshot_name = subprocess.check_output(["zenity", "--entry", "--title=Jarvis-Screen", "--text=Nom déjà utilisé"])
				screenshot_name = screenshot_name[:-1]
			else:
				break

		os.rename(
			os.path.join(self.current, self.userdir, "img", "%s.png" % screenshot_id),
			os.path.join(self.current, self.usedir, "img", "%s.png" % screenshot_name)
		)

	def notes(self):
		os.system('vim %s/%s/%s' % (self.current, self.userdir, self.notes_))



#!/usr/bin/python
# -*- coding: utf-8 -*-

import argparse
import sys


class JarvisParser(argparse.ArgumentParser):
	"""
	Slightly modified argument parser dedicated to Jarvis
	"""
	def __init__(self):
		super(JarvisParser, self).__init__(self, 'Jarvis')



import argparse
import sys
import os
from os.path import join as buildpath
import ConfigParser
import subprocess
from pentest import __version__
from pentest.parser import JarvisParser
from pentest.manager import JarvisManager


def main(args=None):
	"""
	Main entry point for Jarvis

	Creates a user-level .jarvis directory for managing a couple of configuration
	options
	"""
	if not os.path.isfile('/etc/jarvis.conf'):
		print '[!] File /etc/jarvis.conf does not exist, please create it'
		print 'Sample configuration file below, please make sure the pentest_history file will be writable by Jarvis users'
		print
		print "[jarvis]"
		print "pentests_history = /home/marauder/.pentests_history"
		print "user_directory = user"
		print "output_directory = records"
		print "pentest_tree = ["
		print '	"%(user_directory)s",'
		print '	"%(user_directory)s/img",'
		print '	"%(user_directory)s/scripts",'
		print '	"%(user_directory)s/records"'
		print '	]'
		print "notesfiles = notes.txt"

		sys.exit(0)

	manager = JarvisManager()

	# Execute user-supplied action
	args = manager.parser.parse_args()
	if args.command == 'start':
		manager.start(args.directory)
	elif args.command == 'edit':
		manager.edit(args.image)
	elif args.command == "stop":
		manager.stop()
	elif args.command == "init":
		manager.init()
	elif args.command == "screenshot":
		manager.screenshot()
	elif args.command == "notes":
		manager.notes()
	elif args.command == "img":
		manager.img()


if __name__ == '__main__':
	main()



# -*- coding: utf-8 -*-
#
# This file were created by Python Boilerplate. Use boilerplate to start simple
# usable and best-practices compliant Python projects.
#
# Learn more about it at: http://github.com/fabiommendes/python-boilerplate/
#

import os
import codecs
from setuptools import setup, find_packages
import pkgutil
import sys
from setuptools.command.develop import develop
from setuptools.command.install import install


class PostDevelopCommand(develop):
	"""Post-installation for development mode."""
	def run(self):
		develop.run(self)


class PostInstallCommand(install):
	"""Post-installation for installation mode."""
	def run(self):
		install.run(self)

# Save version and author to __meta__.py
version = open('VERSION').read().strip()
dirname = os.path.dirname(__file__)
path = os.path.join(dirname, 'src', u'pentest', '__meta__.py')
meta = '''# Automatically created. Please do not edit.
__version__ = '%s'
__author__ = u'b0z'
''' % version
with open(path, 'w') as F:
	F.write(meta)

hooks_path = os.path.dirname(os.path.abspath(__file__)) + '/src/pentest/hooks'
jarvis_entry_points = {
	'console_scripts': [
		'pentest = pentest.__main__:main'
	]
}

for _, name, _ in pkgutil.iter_modules([hooks_path]):
	jarvis_entry_points['console_scripts'].append('{hook} = pentest.hooks.{hook}:main'.format(hook=name))

setup(
	# Basic info
	name=u'jarvis-pentest',
	version=version,
	author='b0z',
	author_email='bastien@faure.io',
	url='',
	description='A short description for your project.',
	long_description=codecs.open('README.rst', 'rb', 'utf8').read(),
	cmdclass={
		'develop': PostDevelopCommand,
		'install': PostInstallCommand,
	},

	# Classifiers (see https://pypi.python.org/pypi?%3Aaction=list_classifiers)
	classifiers=[
		'Development Status :: 4 - Beta',
		'Intended Audience :: Information Technology',
		'License :: OSI Approved :: MIT License',
		'Operating System :: POSIX :: Linux',
		'Programming Language :: Python',
		'Programming Language :: Python :: 2.7',
		'Programming Language :: Python :: 2 :: Only',
		'Topic :: Software Development :: Libraries',
	],

	# Packages and dependencies
	package_dir={'': 'src'},
	packages=find_packages('src'),
	install_requires=[
	],
	extras_require={
		'dev': [
			'python-boilerplate[dev]',
		],
	},

	# Other configurations
	zip_safe=False,
	platforms='any',
	download_url = 'https://github.com/BastienFaure/jarvis/archive/0.1.tar.gz',

	# entry points
	entry_points=jarvis_entry_points,
)
